from pipeline import main
import pytest


@pytest.fixture
def exec():
    input = [
        "-c",
        "example",
        "-v",
        "pipeline",
        "--dryrun",
        "rest",
        "of",
        "args",
        "--FOO=bar",
        "more",
    ]
    exec = main.parse_arguments(input)
    return exec


def test_set_config_directory(exec):
    assert exec.config_dir.endswith("example")
    assert exec.config_dir.startswith("/home")


def test_detects_flags_before_pipeline(exec):
    assert exec.verbose


def test_ignores_flags_after_pipeline(exec):
    assert exec.dry_run == False


def test_detects_pipeline(exec):
    assert exec.variables["PIPELINE_NAME"] == "pipeline"


def test_detect_rest_of_positionals(exec):
    rest_of_args = ["--dryrun", "rest", "of", "args", "--FOO=bar", "more"]
    expected = {
        "_0": "rest of args more",
        "_1": "rest",
        "_2": "of",
        "_3": "args",
        "_4": "more",
    }
    actual = main.parse_rest_of_args(rest_of_args)
    assert expected.items() < actual.items()


def test_detect_rest_of_name_args(exec):
    rest_of_args = ["--dryrun", "rest", "of", "args", "--FOO=bar", "more"]
    expected = {
        "dryrun": "",
        "FOO": "bar",
    }
    actual = main.parse_rest_of_args(rest_of_args)
    assert expected.items() < actual.items()



from pipeline import main
from os.path import expanduser
import pytest


def test_tilde_expansion():
    result = main.shellexpansion("~/a/b/c", {})
    expected = expanduser("~") + "/a/b/c"
    assert result == expected


def test_single_variable():
    variables = {"FOO": "bar"}
    result = main.shellexpansion("$FOO", variables)
    expected = "bar"
    assert result == expected


def test_curly_variable():
    variables = {"FOO": "bar"}
    result = main.shellexpansion("${FOO}", variables)
    expected = "bar"
    assert result == expected


def test_curly_defaulted_variable():
    variables = {"FOO": "bar"}
    result = main.shellexpansion("${FOO}_${NOPE:yes}", variables)
    expected = "bar_yes"
    assert result == expected


def test_full_expansion():
    variables = {"A": "AA", "B": "BBB"}
    result = main.shellexpansion("~/$A/${B}/${C:C}", variables)
    home = expanduser("~")
    expected = f"{home}/AA/BBB/C"
    assert result == expected


def test_missing_variable_throws_exception():
    with pytest.raises(main.VariableNotSet):
        main.shellexpansion("$NotSet", {})


def test_shellexpansion_dict():
    test_input = {"static": "This should not change.", "dynamic": "${A}_$B"}
    variables = {"A": "AA", "B": "BBB"}
    result = main.shellexpansion_dict(test_input, variables)
    expected = {"static": "This should not change.", "dynamic": "AA_BBB"}
    assert result == expected



import os

from pipeline import main
import pytest

CONFIG_DIR = "example"


def test_load_pipeline():
    pipelines = main.Pipelines(f"{CONFIG_DIR}/pipelines.yaml")
    assert pipelines.tasks("structural") != []


@pytest.fixture
def exec():
    executor = main.Executor.from_config_dir(CONFIG_DIR)
    executor.variables["FOO"] = "bar"
    return executor


def test_executor_load_variables(exec):
    before = dict(exec.variables)
    exec.load_variables("functional")
    after = dict(exec.variables)
    assert len(before) < len(after)


def test_executor_load_variables_has_expansion(exec):
    exec.load_variables("functional")
    raw = "$HOME/XNAT_BUILD_DIR/$USER"
    actual = exec.variables["XNAT_PBS_JOBS_BUILD_DIR"]
    assert raw != actual


def test_executor_load_variables_bad_argument(exec):
    with pytest.raises(TypeError):
        exec.load_variables(None)


def test_executor_load_variables_nonexistent_set_throws_error(exec):
    with pytest.raises(main.VariableSetNotDefined):
        exec.load_variables("not exist")


def test_generate_file_filepath_default_variable_set(exec):
    before = dict(exec.variables)
    exec.generate_file(
        {
            "template": "pbs_head.jinja2",
            "filepath": "~/delete_me.sh",
            "variable": "OUT",
        },
        dryrun=True,
    )
    after = dict(exec.variables)

    assert "OUT" not in before
    assert "OUT" in after


def test_shellexpanded_generated_filepath(exec):
    exec.generate_file(
        {
            "template": "pbs_head.jinja2",
            "filepath": "~/delete_me.$FOO.sh",
        },
        dryrun=True,
    )
    filepath = exec.variables["OUTPUT_FILE"]
    assert filepath.endswith("delete_me.bar.sh")
    assert filepath.startswith("/home/")


def test_generated_output_content(exec):
    result = exec.generate_file(
        {
            "template": "pbs_head.jinja2",
            "filepath": "~/delete_me.$USER.sh",
        },
        dryrun=True,
    )
    expected = "#PBS -S /bin/bash\n#PBS -l nodes=1:ppn=1,walltime=4:00:00,mem=4gb\n"
    assert expected in result


def test_saving_generated_file(exec):
    expected_content = exec.generate_file(
        {
            "template": "pbs_head.jinja2",
            "filepath": "delete_me.sh",
            "variable": "script_path",
        }
    )
    expected_excerpt = (
        "#PBS -S /bin/bash\n#PBS -l nodes=1:ppn=1,walltime=4:00:00,mem=4gb\n"
    )

    filepath = exec.variables["script_path"]
    assert os.path.exists(filepath)

    with open(filepath, "r") as fd:
        actual_content = fd.read()

    assert expected_excerpt in actual_content
    assert expected_content == actual_content

    # clean up generated file
    os.remove(filepath)
    assert not os.path.exists(filepath)


def test_call_function(exec):
    exec.variables["SUBJECT"] = "proj:x:y:z"
    expected = {
        "PROJECT": "proj",
        "SUBJECT_ID": "x",
        "SUBJECT_CLASSIFIER": "y",
        "SUBJECT_EXTRA": "z",
        "SESSION": "x_y",
    }
    actual = exec.function("split_subject")
    all_vars = exec.variables
    assert expected == actual
    # check that expected items are a subset of all_vars
    assert expected.items() <= all_vars.items()


def test_calling_function_throws_error(exec):
    exec.variables["SUBJECT"] = "missing_stuff:x"
    with pytest.raises(ValueError):
        actual = exec.function("split_subject")


def test_set_variables(exec):
    before = dict(exec.variables)
    exec.set_variables({"USER": "New User Account"})
    after = dict(exec.variables)
    assert before["USER"] != after["USER"]


@pytest.fixture
def exec_with_pipeline(exec):
    exec.variables["PIPELINE_NAME"] = "structural"
    return exec


def test_execute_pipeline(exec_with_pipeline):
    exec = exec_with_pipeline
    main.execute_pipeline(exec)



import importlib.util
import pytest
from jinja2 import TemplateNotFound, Environment, FileSystemLoader

from pipeline import main

CONFIG_DIR = "example"


def test_load_existing_yaml():
    pipelines = main.load_yaml(f"{CONFIG_DIR}/pipelines.yaml")
    assert type(pipelines) == dict
    assert len(pipelines) == 5


def test_load_nonexisting_yaml():
    nonexistant = main.load_yaml(f"{CONFIG_DIR}/nonexistent.yaml")
    assert type(nonexistant) == dict
    assert len(nonexistant) == 0


def test_with_default_value_not_added_to_result():
    function_name = "example_function"
    variables = {"REQUIRED": True}
    f = main.Functions(f"{CONFIG_DIR}/functions.py")
    actual = f.execute(function_name, variables)
    expected = {"SET_THIS_VARIABLE": "1", "DEFAULTED": "default"}
    assert actual == expected


def test_function_with_required_and_defaulted_value():
    function_name = "example_function"
    variables = {"REQUIRED": True, "WITH_DEFAULT_VALUE": "new value"}
    f = main.Functions(f"{CONFIG_DIR}/functions.py")
    actual = f.execute(function_name, variables)
    expected = {
        "SET_THIS_VARIABLE": "1",
        "DEFAULTED": "new value",
    }
    assert actual == expected


def test_missing_function():
    function_name = "nonexistent_function"
    variables = {"REQUIRED": True}
    f = main.Functions(f"{CONFIG_DIR}/functions.py")
    with pytest.raises(main.FunctionNotDefined):
        f.execute(function_name, variables)


def test_missing_variables():
    function_name = "example_function"
    variables = {}
    f = main.Functions(f"{CONFIG_DIR}/functions.py")
    with pytest.raises(main.VariableNotSet):
        f.execute(function_name, variables)


def test_load_template():
    template_name = "pbs_head.jinja2"
    template_env = main.TemplateEnv(f"{CONFIG_DIR}/templates")
    variables = {"PBS_NODES": "99"}
    results = template_env.render(template_name, variables)
    expected = "#PBS -S /bin/bash\n#PBS -l nodes=99:ppn=1,walltime=4:00:00,mem=4gb\n"
    assert results == expected


def test_load_template_doesnt_exist():
    template_name = "nonexistent.jinja2"
    template_env = main.TemplateEnv(f"{CONFIG_DIR}/templates")
    with pytest.raises(TemplateNotFound):
        template_env.render(template_name, {})


def test_load_template_config_doesnt_exist():
    template_env = main.TemplateEnv(f"{CONFIG_DIR}/nonexistent")
    with pytest.raises(TemplateNotFound):
        template_env.render("_unimportant_.j2", {})



#!/usr/bin/env python
from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="prunner",
    version="0.1.2",
    author="Moises Baltazar Garcia",
    author_email="me@moisesb.com",
    description="This runs pipelines configured in YAML",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mobalt/pipeline-runner",
    packages=find_packages(),
    # packages=["pipeline"],
    python_requires=">=3.5",
    install_requires=["PyYAML", "Jinja2"],
    entry_points={
        "console_scripts": [
            "prunner = prunner.main:main",
        ],
    },
    keywords="prunner",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: System Administrators",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 3",
        "Topic :: System :: Systems Administration",
        "Topic :: Software Development :: Build Tools",
    ],
    test_suite="tests",
    tests_require=["pytest"],
)
